package com.persagy.proxy.object.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.proxy.adm.service.impl.AbstractAdmBaseServiceImpl;
import com.persagy.proxy.object.model.AdmFloor;
import com.persagy.proxy.adm.request.AdmQueryCriteria;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.object.service.IAdmFloorService;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * ADM 楼层 实现类
 * @author lvxy
 * @date 2021/8/16
 */
@Service
public class AdmFloorServiceImpl extends AbstractAdmBaseServiceImpl<AdmFloor> implements IAdmFloorService {
    @Autowired
    private ObjectMapper objectMapper;

    public static final Boolean QUERY = true;
    public static final Boolean CREAT_UPDATE = false;

    @Override
    public AdmResponse doQuery(InstanceUrlParam context, AdmQueryCriteria request, Class<AdmFloor> clazz) {
        AdmResponse response = super.doQuery(context, request, clazz);
        // 处理properteies 特殊属性
        ensureSpecialValues(context, (List<AdmFloor>) response.getContent(), QUERY);
        return response;
    }
    @Override
    public AdmResponse doQueryOutLine(InstanceUrlParam context, AdmQueryCriteria request, Class<AdmFloor> clazz) {
        request.setProjection(null);
        List<String> list = new ArrayList<>();
        list.add("id");
        list.add("name");
        list.add("localId");
        list.add("localName");
        list.add("outline");
        list.add("floorSequenceId");
        list.add("modelId");
        list.add("infos");
        list.add("properties");
        request.setProjection(list);
        AdmResponse rs = doQuery(context, request, clazz);
        List<AdmFloor> vos = (List<AdmFloor>) rs.getContent();
        ensureSpecialValues(context, vos, QUERY);
        rs.setContent(vos);
        return rs;
    }

    @Override
    public List<AdmFloor> queryFloorInBuilding(InstanceUrlParam context, String buildingId) {
        AdmQueryCriteria request = new AdmQueryCriteria();
        request.setName(AdmFloor.OBJ_TYPE);
        request.resetRelationCond(false, "ArchSubset", "Bd2Fl", buildingId);
        request.setOrders("floorSequenceId desc");
        AdmResponse rs = doQuery(context, request, AdmFloor.class);
        List<AdmFloor> vos = (List<AdmFloor>) rs.getContent();
        ensureSpecialValues(context, vos, QUERY);
        return vos;
    }

    /**
     * 楼层自定义信息点处理 新增，properties 转化为String类型，放到propertiesInfo信息点存储，查询相关，propertiesInfo转为为properties 页面展示
     * @param context
     * @param queryFlag true 查询，false 更新或者新增
     * @return
     */
    @Override
    public void ensureSpecialValues(InstanceUrlParam context, List<AdmFloor> vos, boolean queryFlag) {
        if(CollUtil.isNotEmpty(vos)){
            if(queryFlag){//查询
                vos.stream().forEach(admFloor -> {
                    if(StrUtil.isNotEmpty(admFloor.getPropertiesInfo())){
                        try {
                            ObjectNode properties = (ObjectNode) objectMapper.readTree(admFloor.getPropertiesInfo());
                            admFloor.setProperties(properties);
                            admFloor.setPropertiesInfo(null);
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                });
            }else{//新增或者更新
                vos.stream().forEach(admFloor -> {
                    if(!JSONUtil.isNull(admFloor.getProperties())){
                        admFloor.setPropertiesInfo(admFloor.getProperties().toString());
                        admFloor.setProperties(null);
                    }
                });
            }
        }
    }
}