package com.persagy.proxy.relation.controller;

import com.persagy.proxy.adm.constant.AdmRelationTypeEnum;
import com.persagy.proxy.relation.model.ShInBdList;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.adm.service.IAdmRelationService;
import com.persagy.proxy.adm.utils.AdmContextUtil;
import com.persagy.proxy.common.entity.RelationDTO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * 关系-竖井所在建筑
 * @author zhangwencheng
 */
@Slf4j
@RestController
@RequestMapping("/rel/sh-in-bd")
public class RelationShInBdController {

    @Autowired
    private IAdmRelationService service;

    /**
     * 创建竖井所在建筑关系
     * @param shInBdList
     * @return
     */
    @PostMapping("link-bd")
    public AdmResponse createBd(@RequestBody ShInBdList shInBdList){
        log.info("创建系统所在建筑关系link-bd");
        List<String> buildingIdList = shInBdList.getBuildingIdList();
        String shaftId = shInBdList.getShaftId();
        //检验输入参数
        if(CollectionUtils.isEmpty(buildingIdList)|| StringUtils.isEmpty(shaftId)){
            return AdmResponse.failure("必填项：shaftId（竖井id）、BuildingIdList（建筑id列表）");
        }
        List<RelationDTO> voList = new ArrayList<>();
        String graphCode = AdmRelationTypeEnum.BD2SH_ARCHFORARCH.getGraphCode();
        String relCode = AdmRelationTypeEnum.BD2SH_ARCHFORARCH.getRelCode();
        // 组装关系
        for (String buildingId : buildingIdList) {
            RelationDTO relation = RelationDTO.builder().graphCode(graphCode).relCode(relCode)
                    .objFrom(buildingId).objTo(shaftId).build();
            voList.add(relation);
        }
        service.doSave(AdmContextUtil.toDmpContext(), voList);
        return AdmResponse.success();
    }

}
